// ----------------------------------------------------------------------------

// SystemC main program

// Copyright (C) 2008  Embecosm Limited <info@embecosm.com>

// Contributor Jeremy Bennett <jeremy.bennett@embecosm.com>

// This file is part of the GDB interface to the cycle accurate model of the
// OpenRISC 1000 based system-on-chip, ORPSoC, built using Verilator.

// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.

// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----------------------------------------------------------------------------

// SystemC main program

// $Id: OrpsocMain.cpp 326 2009-03-07 16:47:31Z jeremy $

#include "OrpsocMain.h"

#include "jtagsc.h"

#include "Vorpsoc_fpga_top.h"

#include "OrpsocAccess.h"
#include "TraceSC.h"
#include "ResetSC.h"
#include "Or1200MonitorSC.h"
#include "GdbServerSC.h"


int sc_main (int   argc,
	     char *argv[] )
{
  // Argument handling. There is at most one argument, the port to listen
  // on. Badly formed arguments (i.e where there are no initial digits) will
  // be silently treated as zero, thereby invoking a request for the named
  // Or1ksim service, rather than a specific port number. Any additional
  // arguments are silently ignored.

  int  rspPort = DEFAULT_RSP_PORT;

  if (argc > 1)
    {
      rspPort = atoi (argv[1]);
    }
      
  // Verilator accessor
  OrpsocAccess    *accessor;

  // Modules
  Vorpsoc_fpga_top *orpsoc;		// Verilated ORPSoC
  TraceSC          *trace;		// Drive VCD

  ResetSC          *reset;		// Generate a RESET signal
  Or1200MonitorSC  *monitor;		// Handle l.nop x instructions
  JtagSC           *jtag;		// Generate JTAG signals
  GdbServerSC      *gdbServer;		// Map RSP requests to debug unit

  // Instantiate the Verilator model, VCD trace handler and accessor
  orpsoc     = new Vorpsoc_fpga_top ("orpsoc");
  trace      = new TraceSC ("trace", orpsoc, "v-dump.vcd");
  accessor   = new OrpsocAccess (orpsoc);

  // Instantiate the other SystemC modules
  reset         = new ResetSC ("reset", BENCH_RESET_TIME);
  monitor       = new Or1200MonitorSC ("monitor", accessor);
  jtag          = new JtagSC ("jtag");
  gdbServer     = new GdbServerSC ("gdb-server", FLASH_START, FLASH_END,
				   rspPort, jtag->tapActionQueue);

  // CPU and JTAG clocks. Clocks start low (i.e with negedge)
  sc_time   clkPeriod (BENCH_CLK_HALFPERIOD * 2.0, TIMESCALE_UNIT);
  sc_time   jtagPeriod (JTAG_CLK_HALFPERIOD * 2.0, TIMESCALE_UNIT);

  sc_clock  clk ("clk", clkPeriod, 0.5, SC_ZERO_TIME, false);
  sc_clock  jtagTck ("jtag-clk", jtagPeriod, 0.5, SC_ZERO_TIME, false);

  // Reset signals (both active high and low)
  sc_signal<bool>      rst;
  sc_signal<bool>      rstn;

  // JTAG signals
  sc_signal<bool>      jtagTdi;
  sc_signal<bool>      jtagTdo;
  sc_signal<bool>      jtagTms;
  sc_signal<bool>      jtagTrst;

  sc_signal<bool>      codecLrclk;	// External codec
  sc_signal<bool>      codecMclk;
  sc_signal<bool>      codecSclk;
  sc_signal<bool>      codecSdin;
  sc_signal<bool>      codecSdout;

  sc_signal<bool>      ethCol;		// External Ethernet
  sc_signal<bool>      ethCrs;
  sc_signal<bool>      ethFdsMdint;
  sc_signal<bool>      ethMdc;
  sc_signal<bool>      ethMdi;
  sc_signal<bool>      ethMdo;
  sc_signal<bool>      ethMdoe;

  sc_signal<bool>      ethRxClk;
  sc_signal<bool>      ethRxDv;
  sc_signal<bool>      ethRxEr;
  sc_signal<uint32_t>  ethRxd;

  sc_signal<bool>      ethTxClk;
  sc_signal<bool>      ethTxEn;
  sc_signal<bool>      ethTxEr;
  sc_signal<uint32_t>  ethTxd;

  sc_signal<bool>      ps2ClkI;		// External PS/2 keyboard
  sc_signal<bool>      ps2ClkO;
  sc_signal<bool>      ps2ClkOe;
  sc_signal<bool>      ps2DataI;
  sc_signal<bool>      ps2DataO;
  sc_signal<bool>      ps2DataOe;

  sc_signal<bool>      uartRx;		// External UART
  sc_signal<bool>      uartTx;

  sc_signal<uint32_t>  vgaB;		// External VGA
  sc_signal<uint32_t>  vgaG;
  sc_signal<uint32_t>  vgaR;

  sc_signal<bool>      vgaBlank;
  sc_signal<bool>      vgaHsyncn;
  sc_signal<bool>      vgaPclk;
  sc_signal<bool>      vgaVsyncn;

  // Connect up ORPSoC
  orpsoc->clk (clk);
  orpsoc->rstn (rstn);

  orpsoc->jtag_tck  (jtagTck);		// JTAG interface
  orpsoc->jtag_tdi  (jtagTdi);
  orpsoc->jtag_tdo  (jtagTdo);
  orpsoc->jtag_tms  (jtagTms);
  orpsoc->jtag_trst (jtagTrst);

  orpsoc->codec_lrclk (codecLrclk);    // External codec
  orpsoc->codec_mclk  (codecMclk);
  orpsoc->codec_sclk  (codecSclk);
  orpsoc->codec_sdin  (codecSdin);
  orpsoc->codec_sdout (codecSdout);

  orpsoc->eth_col       (ethCol);      // External Ethernet
  orpsoc->eth_crs       (ethCrs);
  orpsoc->eth_fds_mdint (ethFdsMdint);
  orpsoc->eth_mdc       (ethMdc);
  orpsoc->eth_mdi       (ethMdi);
  orpsoc->eth_mdo       (ethMdo);
  orpsoc->eth_mdoe      (ethMdoe);

  orpsoc->eth_rx_clk (ethRxClk);
  orpsoc->eth_rx_dv  (ethRxDv);
  orpsoc->eth_rx_er  (ethRxEr);
  orpsoc->eth_rxd    (ethRxd);

  orpsoc->eth_tx_clk (ethTxClk);
  orpsoc->eth_tx_en  (ethTxEn);
  orpsoc->eth_tx_er  (ethTxEr);
  orpsoc->eth_txd    (ethTxd);

  orpsoc->ps2_clk_i   (ps2ClkI);      // External PS/2 keyboard
  orpsoc->ps2_clk_o   (ps2ClkO);
  orpsoc->ps2_clk_oe  (ps2ClkOe);
  orpsoc->ps2_data_i  (ps2DataI);
  orpsoc->ps2_data_o  (ps2DataO);
  orpsoc->ps2_data_oe (ps2DataOe);

  orpsoc->uart_rx (uartRx);		// External UART
  orpsoc->uart_tx (uartTx);

  orpsoc->vga_b (vgaB);		// External VGA
  orpsoc->vga_g (vgaG);
  orpsoc->vga_r (vgaR);

  orpsoc->vga_blank  (vgaBlank);
  orpsoc->vga_hsyncn (vgaHsyncn);
  orpsoc->vga_pclk   (vgaPclk);
  orpsoc->vga_vsyncn (vgaVsyncn);

  // Connect up the VCD trace handler
  trace->clk (clk);			// Trace

  // Connect up the SystemC  modules
  reset->clk (clk);			// Reset
  reset->rst (rst);
  reset->rstn (rstn);

  monitor->clk (clk);			// Monitor

  jtag->sysReset (rst);			// Connect up the JTAG
  jtag->tck (jtagTck);
  jtag->tdi (jtagTdi);
  jtag->tdo (jtagTdo);
  jtag->tms (jtagTms);
  jtag->trst (jtagTrst);

  // Tie off signals
  jtagTdi     = 1;			// Tie off the JTAG inputs
  jtagTms     = 1;

  codecSdout  = 0;			// Tie off the codec

  ethCol      = 0;			// Tie off the Ethernet
  ethCrs      = 0;
  ethFdsMdint = 0;
  ethMdi      = 1;

  ethRxEr     = 0;
  ethRxClk    = 0;
  ethRxDv     = 0;
  ethRxd      = 0;

  ethTxClk    = 0;

  ps2ClkI     = 1;			// Tie off the PS/2 keyboard
  ps2DataI    = 1;

  uartRx      = 0;			// Tie off the UART

  // Execute until we stop
  sc_start ();

  // Free memory
  delete gdbServer;
  delete jtag;
  delete monitor;
  delete reset;

  delete accessor;

  delete trace;
  delete orpsoc;

  return 0;

}	/* sc_main() */
