// ----------------------------------------------------------------------------

// SystemC OpenRISC 1000 Debug Unit: implementation

// Copyright (C) 2008  Embecosm Limited <info@embecosm.com>

// Contributor Jeremy Bennett <jeremy.bennett@embecosm.com>

// This file is part of the GDB interface to the cycle accurate model of the
// OpenRISC 1000 based system-on-chip, ORPSoC, built using Verilator.

// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.

// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----------------------------------------------------------------------------

// $Id: DebugUnitSC.cpp 331 2009-03-12 17:01:48Z jeremy $

#include <iostream>
#include <iomanip>

#include "DebugUnitSC.h"
#include "Utils.h"


using sc_core::sc_event;
using sc_core::sc_fifo;
using sc_core::sc_module_name;


//-----------------------------------------------------------------------------
//! Constructor for the Debug Unit

//! This module is entirely subservient to the GDB server. It has no SystemC
//! processes of its own. It provides services via calls to its API.

//! The current scan chain is marked as undefined, and the current JTAG scan
//! chain is maked as undefined.

//! Caches for SPR and memory access are initialized.

//! This makes use of the Embecosm cycle accurate SystemC JTAG interface.

//! @see Embecosm Application Note 5 "Using JTAG with SystemC: Implementation
//!      of a Cycle Accurate Interface"
//!      (http://www.embecosm.com/download/ean5.html)

//! @param[in] name             Name of this module, passed to the parent
//!                             constructor. 
//! @param[in] _tapActionQueue  Pointer to fifo of actions to be performed by
//!                             the JTAG interface
//-----------------------------------------------------------------------------
DebugUnitSC::DebugUnitSC (sc_module_name        name,
			  sc_fifo<TapAction *> *_tapActionQueue) :
  sc_module (name),
  tapActionQueue (_tapActionQueue),
  stallState (UNKNOWN),
  currentScanChain (OR1K_SC_UNDEF)
{
#ifdef NOCACHE
  npcCacheIsValid = false;		// Always cache NPC
#else
  sprCache = new SprCache ();
  memCache = new MemCache ();
#endif

}	// DebugUnitSC ()


//-----------------------------------------------------------------------------
//! Destructor

//! Free up data structures
//-----------------------------------------------------------------------------
DebugUnitSC::~DebugUnitSC ()
{
#ifndef NOCACHE
  delete  memCache;
  delete  sprCache;
#endif

}	// ~DebugUnitSC


//-----------------------------------------------------------------------------
//! Reset the Debug Unit

//! This is just a reset of the JTAG. It is quite possible to reset the debug
//! unit without resetting the whole target.

//! @note Must be called from a SystemC thread, because of the use of wait()
//-----------------------------------------------------------------------------
void
DebugUnitSC::resetDebugUnit ()
{
  sc_event       *done = new sc_event();
  TapActionReset *resetAction;

  // Create and queue the reset action and wait for it to complete
  resetAction = new TapActionReset (done);
  tapActionQueue->write (resetAction);
  wait (*done);

  delete resetAction;
  delete done;

}	// resetDebugUnit ()


//-----------------------------------------------------------------------------
//! Reset the processor

//! Read the RISCOP register, OR in the reset bit and write it back.

//! After reset, the processor is known to be unstalled.
//-----------------------------------------------------------------------------
void      
DebugUnitSC::reset ()
{
  writeRiscop (readRiscop () | RISCOP_RESET);
  stallState = UNKNOWN;

}	// reset ()


//-----------------------------------------------------------------------------
//! Stall the processor

//! Read the RISCOP register, OR in the stall bit and write it back.
//-----------------------------------------------------------------------------
void      
DebugUnitSC::stall ()
{
  writeRiscop (readRiscop () | RISCOP_STALL);
  stallState = STALLED;

}	// stall ()


//-----------------------------------------------------------------------------
//! Unstall the processor

//! Read the RISCOP register, AND out the stall bit and write it back. After
//! this the NPC cache will be invalid.

//! @note Don't be tempted to read back for confirmation. Single stepping
//!       will already have stalled the processor again!
//-----------------------------------------------------------------------------
void      
DebugUnitSC::unstall ()
{
  writeRiscop (readRiscop () & ~RISCOP_STALL);
  stallState  = UNKNOWN;

#ifdef NOCACHE
  npcCacheIsValid = false;		// Always cache NPC
#else
  // Clear the caches
  sprCache->clear ();
  memCache->clear ();
#endif

}	// unstall ()


//-----------------------------------------------------------------------------
//! Report if the processor is stalled.

//! A stalled processor cannot spontaneously "unstall", so if the stallState
//! flag is STALLED, that value is returned. Otherwise the target is
//! interrogated to determine the status.

//! @return  TRUE if the process is known to be stalled
//-----------------------------------------------------------------------------
bool
DebugUnitSC::isStalled ()
{
  if (STALLED == stallState)
    {
      return  true;
    }

  uint32_t  riscop = readRiscop ();
  stallState = (RISCOP_STALL == (riscop & RISCOP_STALL)) ? STALLED : UNKNOWN;

  return  STALLED == stallState;

}	// isStalled ()


//-----------------------------------------------------------------------------
//! Read the value of an OpenRISC 1000 Special Purpose Register

//! First see if we have the value in the cache, and if so return
//! it. Otherwise, select the RISC_DEBUG scan chain and read from JTAG,
//! storing the result in the cache.

//! @param[in]  sprNum  The SPR to read

//! @return  The value of the SPR
//-----------------------------------------------------------------------------
uint32_t  
DebugUnitSC::readSpr (uint16_t  sprNum)
{
  uint32_t  cachedValue;

#ifdef NOCACHE
  // Always check NPC cache
  if ((STALLED == stallState) && (sprNum == SPR_NPC) && npcCacheIsValid)
    {
      return  npcCachedValue;
    }
#else
  // Use any cached value if we are stalled.
  if ((STALLED == stallState) && sprCache->read (sprNum, cachedValue))
    {
      return  cachedValue;		// Already there, no more to do
    }
#endif

  // Read the value
  selectJtagChain (OR1K_SC_RISC_DEBUG);
  cachedValue = readJtagReg (sprNum);

#ifdef NOCACHE
  // Always update the NPC cache
  if ((STALLED == stallState) && (sprNum == SPR_NPC))
    {
      npcCachedValue  = cachedValue;
      npcCacheIsValid = true;
    }
#else
  // Update the cache if we are stalled
  if (STALLED == stallState)
    {
      sprCache->write (sprNum, cachedValue, sprNum == SPR_NPC);
    }
#endif

  return  cachedValue;

}	// readSpr ()


//-----------------------------------------------------------------------------
//! Write the value of an OpenRISC 1000 Special Purpose Register

//! First look to see if we are stalled and the value is cached. If the value
//! has not changed, then we need to no more. Otherwise cache the value prior
//! to writing it.

//! Select the RISC_DEBUG scan chain and write to JTAG

//! @param[in] sprNum  The SPR to write
//! @param[in] value   The value to write
//-----------------------------------------------------------------------------
void      
DebugUnitSC::writeSpr (uint16_t  sprNum,
		       uint32_t  value)
{
#ifdef NOCACHE
  // Always cache the NPC
  if ((STALLED == stallState) && (SPR_NPC == sprNum))
    {
      // Have we already cached this NPC value?
      if (npcCacheIsValid && (value == npcCachedValue))
	{
	  return;
	}
      else
	{
	  npcCachedValue  = value;
	  npcCacheIsValid = true;
	}
    }
#else
  if (STALLED == stallState)
    {
      // Have we already cached this value?
      uint32_t  cachedValue;
      if (sprCache->read (sprNum, cachedValue) &&
	  (value == cachedValue))
	{
	  return;				// Already there, no more to do
	}
      else
	{
	  sprCache->write (sprNum, value, sprNum == SPR_NPC);
	}
    }
#endif

  // Write the SPR
  selectJtagChain (OR1K_SC_RISC_DEBUG);
  writeJtagReg (sprNum, value);

}	// writeSpr ()


//-----------------------------------------------------------------------------
//! AND the contents of an SPR with a value

//! A convenience combination of read and write

//! @param[in] sprNum  The SPR to write
//! @param[in] value   The value to AND into the register
//-----------------------------------------------------------------------------
void      
DebugUnitSC::andSpr (uint16_t  sprNum,
		     uint32_t  value)
{
  writeSpr (sprNum, readSpr (sprNum) & value);

}	// andSpr ()


//-----------------------------------------------------------------------------
//! OR the contents of an SPR with a value

//! A convenience combination of read and write

//! @param[in] sprNum  The SPR to write
//! @param[in] value   The value to OR into the register
//-----------------------------------------------------------------------------
void      
DebugUnitSC::orSpr (uint16_t  sprNum,
		   uint32_t  value)
{
  writeSpr (sprNum, readSpr (sprNum) | value);

}	// orSpr ()


//-----------------------------------------------------------------------------
//! Read a 32-bit word from the OpenRISC 1000 memory

//! Select the WISHBONE scan chain, then write the register. The data is in
//! model endianness and passed on without modification.

//! @todo Provide code to check if the read was from a valid address.

//! @param[in] addr  The address to read from

//! @return  The 32-bit value read
//-----------------------------------------------------------------------------
uint32_t  
DebugUnitSC::readMem32 (uint32_t  addr)
{
  uint32_t  cachedValue;

#ifndef NOCACHE
  // Use any cached value if we are stalled.
  if ((STALLED == stallState) && memCache->read (addr, cachedValue))
    {
      return  cachedValue;		// Already there, no more to do
    }
#endif

  // Read the value
  selectJtagChain (OR1K_SC_WISHBONE);
  cachedValue = readJtagReg (addr);

#ifndef NOCACHE
  // Update the cache if we are stalled
  if (STALLED == stallState)
    {
      memCache->write (addr, cachedValue);
    }
#endif

  return  cachedValue;

}	// readMem32 ()


//-----------------------------------------------------------------------------
//! Write a 32-bit word to the OpenRISC 1000 memory

//! Select the WISHBONE scan chain, then write the register. The data is in
//! model endianness and passed on without modification.

//! @todo Provide code to check if the write was to a valid address.

//! @param[in] addr   The address to write to
//! @param[in] value  The 32-bit value to write

//! @return  True if the write was successful. For now all writes are
//           successful.
//-----------------------------------------------------------------------------
bool      
DebugUnitSC::writeMem32 (uint32_t  addr,
			 uint32_t  value)
{
#ifndef NOCACHE
  if (STALLED == stallState)
    {
      // Have we already cached this value?
      uint32_t  cachedValue;
      if (memCache->read (addr, cachedValue) &&
	  (value == cachedValue))
	{
	  return  true;				// Already there, no more to do
	}
      else
	{
	  memCache->write (addr, value);	// Write for the future
	}
    }
#endif

  // Write the memory
  selectJtagChain (OR1K_SC_WISHBONE);
  writeJtagReg (addr, value);

  return  true;

}	// writeMem32 ()


//-----------------------------------------------------------------------------
//! Read a byte from the OpenRISC 1000 main memory

//! All we can get are 32-bits words, so we have to unpick the value.

//! The underlying 32-bit routines take target endian arguments and return
//! target endian results. We need to convert to host endianness to access the
//! relevant byte.

//! @todo Provide code to check if the read was from a valid address.

//! @note Read bytes from memory mapped devices at your peril!

//! @param[in] addr  The address to read from
//! @return  The byte read
//-----------------------------------------------------------------------------
uint8_t   
DebugUnitSC::readMem8 (uint32_t  addr)
{
  uint32_t  word   = Utils::ttohl (readMem32 (addr & 0xfffffffc));
  uint8_t  *bytes  = (uint8_t *)(&word);
  int       offset = addr & 0x3;

  return  bytes[offset];

}	// readMem8 ()


//-----------------------------------------------------------------------------
//! Write a byte to the OpenRISC 1000 main memory

//! All we can get are 32-bits words, so we have to read the current value and
//! construct the new value to write back.

//! The underlying 32-bit routines take target endian arguments and return
//! target endian results. We need to convert to host endianness to alter the
//! relevant byte.

//! @note Write bytes to memory mapped devices at your peril!

//! @todo Provide code to check if the write was to a valid address.

//! @param[in] addr   The address to write to
//! @param[in] value  The byte to write

//! @return  True if the write was successful. For now all writes are
//           successful.
//-----------------------------------------------------------------------------
bool      
DebugUnitSC::writeMem8 (uint32_t  addr,
			uint8_t   value)
{
  uint32_t  currWord  = Utils::ttohl (readMem32 (addr & 0xfffffffc));
  uint8_t  *currBytes = (uint8_t *)(&currWord);
  int       offset    = addr & 0x3;

  currBytes[offset] = value;

  return  writeMem32 (addr & 0xfffffffc, Utils::htotl (currWord));

}	// writeMem8 ()


//-----------------------------------------------------------------------------
//! Get the RISCOP control register

//! Convenience function. Select the REGISTER scan chain, read the current
//! value.

//! @return  The value in the RISCOP register
//-----------------------------------------------------------------------------
uint32_t     
DebugUnitSC::readRiscop ()
{
  selectJtagChain (OR1K_SC_REGISTER);
  return readJtagReg (OR1K_RSC_RISCOP);

}	// readRiscop ()


//-----------------------------------------------------------------------------
//! Set the RISCOP control register

//! Convenience function. Select the REGISTER scan chain, write the new value.

//! @param[in] value  The value to write into the RISCOP register
//-----------------------------------------------------------------------------
void
DebugUnitSC::writeRiscop (uint32_t  value)
{
  selectJtagChain (OR1K_SC_REGISTER);
  writeJtagReg (OR1K_RSC_RISCOP, value);

}	// writeRiscop ()


//-----------------------------------------------------------------------------
//! Select an OpenRISC 1000 scan chain

//! Built on top of the JTAG commands to shift registers
//! We only do something if the scan chain needs to be changed.
//! - Shift-IR the CHAIN_SELECT instruction
//! - Shift-DR the specified chain
//! - Shift-IR the DEBUG instruction

//! @note The ORPSoC data register is one bit longer than needed. This final
//!       (most significant bit) is ignored. This is an apparent bug in the
//!       debug interface.

//! @note Must be called from a SystemC thread, because of the use of wait()

//! @param[in] chain  The desired scan chain
//-----------------------------------------------------------------------------
void
DebugUnitSC::selectJtagChain (int chain)
{
  if (chain == currentScanChain)
    {
      return;
    }
  else
    {
      currentScanChain = chain;
    }

  sc_event        *done = new sc_event();
  TapActionIRScan *iRScan;
  TapActionDRScan *dRScan;

  // Create and queue the IR-Scan action for CHAIN_SELECT (no CRC)
  iRScan = new TapActionIRScan (done, CHAIN_SELECT_IR, JTAG_IR_LEN);
  tapActionQueue->write (iRScan);
  wait (*done);

  delete iRScan;

  // Create and queue the DR-Scan action for the specified chain (which we
  // know will fit into 64 bits)
  uint64_t  chainReg ;
  clearBits (&chainReg, CHAIN_DR_LEN);
  packBits (&chainReg, CHAIN_OPCODE_OFF, CHAIN_OPCODE_LEN, chain);
  uint8_t  crc = crc8 (&chainReg, CHAIN_CRC_OFF);
  packBits (&chainReg, CHAIN_CRC_OFF, CHAIN_CRC_LEN, crc);

  dRScan = new TapActionDRScan (done, chainReg, CHAIN_DR_LEN);
  tapActionQueue->write (dRScan);
  wait (*done);

  delete dRScan;

  // Create and queue the IR-Scan action for DEBUG (no CRC)
  iRScan      = new TapActionIRScan (done, DEBUG_IR, JTAG_IR_LEN);
  tapActionQueue->write (iRScan);
  wait (*done);

  delete iRScan;
  delete done;

}	// selectJtagChain()
  

//-----------------------------------------------------------------------------
//! Read an OpenRISC 1000 JTAG register

//! Built on top of the JTAG commands to shift registers
//! - Shift-DR the specified address with R/W field unset
//! - read out the data shifted out.

//! DR register fields depend on the scan chain in use. For SC_REGISTER:
//! -   [4:0] Address to read from
//! -     [5] 0 indicating read
//! -  [37:6] Unused
//! - [45:38] CRC (CRC-8-ATM)
//! - [46:46] Unused

//! For SC_RISC_DEBUG (i.e. SPRs) and SC_WISHBONE:
//! -  [31:0] Address to read from
//! -    [32] 0 indicating read
//! - [64:33] unused
//! - [72:65] CRC (CRC-8-ATM)
//! - [73:73] Unused

//! A single Scan-DR loop is sufficient for SC_REGISTER and
//! SC_RISC_DEBUG. However depending on the JTAG clock speed and memory access
//! speed, a second read may be needed for Wishbone.
//!
//! A polished implementation would actually insert a pause mid-read for
//! Wishbond, but that will have to wait for a future implementation of the
//! JTAG interface. So for now Wishbone accesses get a second read.

//! @note Must be called from a SystemC thread, because of the use of wait()

//! @param[in] addr  The address of the register

//! @return  The register value read
//-----------------------------------------------------------------------------
uint32_t
DebugUnitSC::readJtagReg (uint32_t  addr)
{
  int  drLen;			// Size of the data register
  int  addrOff;			// Offsets and sizes of fields in the DR
  int  addrLen;
  int  dataOff;
  int  dataLen;
  int  crcOff;
  int  crcLen;
   
  // Determine the fields in the data register being read
  switch (currentScanChain)
    {
    case OR1K_SC_RISC_DEBUG:
      drLen   = RISC_DEBUG_DR_LEN;
      addrOff = RISC_DEBUG_ADDR_OFF;
      addrLen = RISC_DEBUG_ADDR_LEN;
      dataOff = RISC_DEBUG_DATA_OFF;
      dataLen = RISC_DEBUG_DATA_LEN;
      crcOff  = RISC_DEBUG_CRC_OFF;
      crcLen  = RISC_DEBUG_CRC_LEN;
      break;

    case OR1K_SC_REGISTER:
      drLen   = REGISTER_DR_LEN;
      addrOff = REGISTER_ADDR_OFF;
      addrLen = REGISTER_ADDR_LEN;
      dataOff = REGISTER_DATA_OFF;
      dataLen = REGISTER_DATA_LEN;
      crcOff  = REGISTER_CRC_OFF;
      crcLen  = REGISTER_CRC_LEN;
      break;

    case OR1K_SC_WISHBONE:
      drLen   = WISHBONE_DR_LEN;
      addrOff = WISHBONE_ADDR_OFF;
      addrLen = WISHBONE_ADDR_LEN;
      dataOff = WISHBONE_DATA_OFF;
      dataLen = WISHBONE_DATA_LEN;
      crcOff  = WISHBONE_CRC_OFF;
      crcLen  = WISHBONE_CRC_LEN;
      break;
    }

  // Initialize the register fields
  uint64_t *dRegIn  = new uint64_t [(drLen + 63) / 64];
  uint64_t *dRegOut = new uint64_t [(drLen + 63) / 64];

  clearBits (dRegIn, drLen);
  packBits (dRegIn, addrOff, addrLen, addr);
  packBits (dRegIn, crcOff, crcLen, crc8 (dRegIn, crcOff));

  // Allocate a SystemC completion event
  sc_event *done  = new sc_event();

  // Loop until CRCs match. Note that the CRC written back is displaced by 1
  // (hence the crcOff + 1). This appears to be a bug in debug unit Verilog
  do
    {
      TapActionDRScan *dRScan = new TapActionDRScan (done, dRegIn, drLen);
      tapActionQueue->write (dRScan);
      wait (*done);
      dRScan->getDRegOut (dRegOut);
      delete dRScan;
    }
  while (crc8 (dRegOut, crcOff) != unpackBits (dRegOut, crcOff + 1, crcLen));

  // If this is one of the CPU control registers (REGISTER scan chain), then a
  // single read is all it takes.
  if ((OR1K_SC_REGISTER == currentScanChain))
    {
      uint32_t  res = unpackBits (dRegOut, dataOff, dataLen);
      delete [] dRegIn;
      delete [] dRegOut;
      delete  done;
      return  res;
//       return  unpackBits (dRegOut, dataOff, dataLen);
    }

  // For WISHBONE and SPRs a second read is required to access the register. A
  // more advanced implementation could take advantage of the JTAG pause
  // feature.
  do
    {
      TapActionDRScan *dRScan = new TapActionDRScan (done, dRegIn, drLen);
      tapActionQueue->write (dRScan);
      wait (*done);
      dRScan->getDRegOut (dRegOut);
      delete dRScan;
    }
  while (crc8 (dRegOut, crcOff) != unpackBits (dRegOut, crcOff + 1, crcLen));

  // All done
  uint32_t  res = unpackBits (dRegOut, dataOff, dataLen);
  delete [] dRegIn;
  delete [] dRegOut;
  delete  done;
  return  res;
//   return  unpackBits (dRegOut, dataOff, dataLen);

}	// readJtagReg ()


//-----------------------------------------------------------------------------
//! Write an OpenRISC 1000 JTAG register

//! Built on top of the JTAG commands to shift registers
//! - Shift-DR the specified address with R/W field set and data to write

//! DR register fields depend on the scan chain in use. For SC_REGISTER:
//! -   [4:0] Address to write to
//! -     [5] 1 indicating write
//! -  [37:6] Value to write
//! - [45:38] CRC (CRC-8-ATM)
//! - [46:46] Unused

//! For SC_RISC_DEBUG (i.e. SPRs) and SC_WISHBONE:
//! -  [31:0] Address to write to
//! -    [32] 1 indicating write
//! - [64:33] Value to write
//! - [72:65] CRC (CRC-8-ATM)
//! - [73:73] Unused

//! @note Must be called from a SystemC thread, because of the use of wait()

//! @param[in] addr  The address of the register
//! @param[in] data  The register data to write
//-----------------------------------------------------------------------------
void
DebugUnitSC::writeJtagReg (uint32_t  addr,
			   uint32_t  data)
{
  int  drLen;			// Size of the data register
  int  addrOff;			// Offsets and sizes of fields in the DR
  int  addrLen;
  int  rwOff;
  int  rwLen;
  int  dataOff;
  int  dataLen;
  int  crcOff;
  int  crcLen;

  // Determine the fields in the data register being read
  switch (currentScanChain)
    {
    case OR1K_SC_RISC_DEBUG:
      drLen   = RISC_DEBUG_DR_LEN;
      addrOff = RISC_DEBUG_ADDR_OFF;
      addrLen = RISC_DEBUG_ADDR_LEN;
      rwOff   = RISC_DEBUG_RW_OFF;
      rwLen   = RISC_DEBUG_RW_LEN;
      dataOff = RISC_DEBUG_DATA_OFF;
      dataLen = RISC_DEBUG_DATA_LEN;
      crcOff  = RISC_DEBUG_CRC_OFF;
      crcLen  = RISC_DEBUG_CRC_LEN;
      break;

    case OR1K_SC_REGISTER:
      drLen   = REGISTER_DR_LEN;
      addrOff = REGISTER_ADDR_OFF;
      addrLen = REGISTER_ADDR_LEN;
      rwOff   = REGISTER_RW_OFF;
      rwLen   = REGISTER_RW_LEN;
      dataOff = REGISTER_DATA_OFF;
      dataLen = REGISTER_DATA_LEN;
      crcOff  = REGISTER_CRC_OFF;
      crcLen  = REGISTER_CRC_LEN;
      break;

    case OR1K_SC_WISHBONE:
      drLen   = WISHBONE_DR_LEN;
      addrOff = WISHBONE_ADDR_OFF;
      addrLen = WISHBONE_ADDR_LEN;
      rwOff   = WISHBONE_RW_OFF;
      rwLen   = WISHBONE_RW_LEN;
      dataOff = WISHBONE_DATA_OFF;
      dataLen = WISHBONE_DATA_LEN;
      crcOff  = WISHBONE_CRC_OFF;
      crcLen  = WISHBONE_CRC_LEN;
      break;
    }

  // Initialize the register fields
  uint64_t *dRegIn  = new uint64_t [(drLen + 63) / 64];
  uint64_t *dRegOut = new uint64_t [(drLen + 63) / 64];

  clearBits (dRegIn, drLen);
  packBits (dRegIn, addrOff, addrLen, addr);
  packBits (dRegIn, rwOff, rwLen, 1);
  packBits (dRegIn, dataOff, dataLen, data);
  packBits (dRegIn, crcOff, crcLen, crc8 (dRegIn, crcOff));

  // Prepare the action, queue it and wait for it to complete
  sc_event        *done   = new sc_event();
  TapActionDRScan *dRScan = new TapActionDRScan (done, dRegIn, drLen);

  tapActionQueue->write (dRScan);
  wait (*done);

  delete [] dRegIn;
  delete dRScan;
  delete done;

}	// writeJtagReg ()


//-----------------------------------------------------------------------------
//! Clear the bits in a data register

//! We always clear whole 64-bit words, not just the minimum number of
//! bytes. It saves all sorts of confusion when debugging code.

//! @note It is the caller's responsibility to make sure the date register
//!       array is large enough.

//! @param[in,out] regArray  The data register to clear
//! @param[in]     regBits  Size of the data register (in bits)
//-----------------------------------------------------------------------------
void
DebugUnitSC::clearBits (uint64_t  regArray[],
			int       regBits)
{
  memset ((char *)regArray, 0, ((regBits + 63) / 64) * 8);

}	// clearBits ()


//-----------------------------------------------------------------------------
//! Set a bit field in a data register

//! The field is cleared, the supplied value masked and then ored into the
//! vector.

//! @note It is the caller's responsibility to make sure the date register
//!       array is large enough.

//! @param[in,out] regArray     The data register
//! @param[in]     fieldOffset  Start of the field (in bits)
//! @param[in]     fieldBits    Size of the field (in bits)
//! @param[in]     fieldVal     Value to set in the field
//-----------------------------------------------------------------------------
void
DebugUnitSC::packBits (uint64_t  regArray[],
		       int       fieldOffset,
		       int       fieldBits,
		       uint64_t  fieldVal)
{
  fieldVal &= (1ULL << fieldBits) - 1ULL;	// Mask the supplied value

  int  startWord =  fieldOffset / 64;
  int  endWord   = (fieldOffset + fieldBits - 1) / 64;

  fieldOffset = fieldOffset % 64;		// Now refers to target word

  // Deal with the startWord. Get enough bits for the mask and put them in the
  // right place
  uint64_t startMask   = ((1ULL << fieldBits) - 1ULL) << fieldOffset;

  regArray[startWord] &= ~startMask;
  regArray[startWord] |= fieldVal << fieldOffset;

  // If we were all in one word, we can give up now.
  if (startWord == endWord)
    {
      return;
    }

  // Deal with the endWord. Get enough bits for the mask. No need to shift
  // these up - they're always at the bottom of the word
  int       bitsToDo = (fieldOffset + fieldBits) % 64;
  uint64_t  endMask  = (1ULL << bitsToDo) - 1ULL;

  regArray[endWord] &= ~endMask;
  regArray[endWord] |= fieldVal >> (fieldBits - bitsToDo);

}	// packBits ()


//-----------------------------------------------------------------------------
//! Extract a bit field from a data register

//! The field is cleared, the supplied value masked and then ored into the
//! vector.

//! @note It is the caller's responsibility to make sure the date register
//!       array is large enough.

//! @param[in,out] regArray     The data register
//! @param[in]     fieldOffset  Start of the field (in bits)
//! @param[in]     fieldBits    Size of the field (in bits)

//! @return  The value in the field
//-----------------------------------------------------------------------------
uint64_t
DebugUnitSC::unpackBits (uint64_t  regArray[],
			 int       fieldOffset,
			 int       fieldBits)
{
  int  startWord = fieldOffset / 64;
  int  endWord   = (fieldOffset + fieldBits - 1) / 64;

  fieldOffset = fieldOffset % 64;		// Now refers to target word

  // Deal with the startWord. Get enough bits for the mask and put them in the
  // right place
  uint64_t  startMask = ((1ULL << fieldBits) - 1ULL) << fieldOffset;
  uint64_t  res       = (regArray[startWord] & startMask) >> fieldOffset;

  // If we were all in one word, we can give up now.
  if (startWord == endWord)
    {
      res &= (1ULL << fieldBits) - 1ULL;	// Mask off any unwanted bits
      return  res;
    }

  // Deal with the endWord. Get enough bits for the mask. No need to shift
  // these up - they're always at the bottom of the word
  int       bitsToDo = (fieldOffset + fieldBits) % 64;
  uint64_t  endMask  = (1ULL << bitsToDo) - 1ULL;

  res = res | ((regArray[endWord] & endMask) << (fieldBits - bitsToDo));
  res &= (1ULL << fieldBits) - 1ULL;		// Mask off any unwanted bits
  return  res;

}	// unpackBits ()


//-----------------------------------------------------------------------------
//! Compute CRC-8-ATM

//! The data is in an array of uint64_t, for which we use the first size bits
//! to compute the CRC.

//! @Note I am using the same algorithm as the ORPSoC debug unit, but I
//!       believe its function is broken! I don't believe the data bit should
//!       feature in the computation of bits 2 & 1 of the new CRC.

//! @Note I've realized that this is an algorithm for LSB first, so maybe it
//!       is correct!

//! @param dataArray  The array of data whose CRC is desired
//! @param size       The number of bits in the data
//-----------------------------------------------------------------------------
uint8_t
DebugUnitSC::crc8 (uint64_t  dataArray[],
		   int       size)
{
  uint8_t    crc = 0;

  for (int  i = 0; i < size; i++)
    {
      uint8_t  d       = (dataArray[i / 64] >> (i % 64)) & 1;
      uint8_t  oldCrc7 = (crc >> 7) & 1;
      uint8_t  oldCrc1 = (crc >> 1) & 1;
      uint8_t  oldCrc0 = (crc >> 0) & 1;
      uint8_t  newCrc2 = d ^ oldCrc1 ^ oldCrc7;		// Why d?
      uint8_t  newCrc1 = d ^ oldCrc0 ^ oldCrc7;		// Why d?
      uint8_t  newCrc0 = d ^ oldCrc7;

      crc = ((crc << 1) & 0xf8) | (newCrc2 << 2) | (newCrc1 << 1) | newCrc0;
    }

  return crc;

}	// crc8 ()
